const express = require('express')
const path = require('path')
const app = express()
const logger = require('morgan')
const bodyParser = require('body-parser')
const moment = require('moment')
const cors = require('cors')
const mongoose = require('mongoose')
// const router = express.router()
// TodosModels
var Todos = require('./model/todos');

//資料庫mLab連線設定
//mongodb://<dbuser>:<dbpassword>@ds129166.mlab.com:29166/todos
//let dbURI = 'mongodb://ncuelan:ncuelan@ds147421.mlab.com:47421/ncuelan';
let dbURI = 'mongodb://todosuser:todosuser@ds129166.mlab.com:29166/todos';


//資料庫連線是否成功
mongoose.connect(dbURI);

mongoose.connection.on('connected', () => {
  console.log(`Mongoose connected to ${dbURI}`);
});
mongoose.connection.on('error', err => {
  console.log('Mongoose connection error:', err);
});
mongoose.connection.on('disconnected', () => {
  console.log('Mongoose disconnected');
});


//建立靜態資料夾，express內建的中介函式
app.use(express.static(path.join(__dirname, 'public'))); //存放靜態資料
//app.use(express.static(path.join(__dirname, 'dist')));//angular程式所在位置

//建立中介函式

app.use(logger('dev'));
app.use(bodyParser.json());
app.use(bodyParser.urlencoded({
  extended: false
}));
app.use(cors());


// app.get('/', function (req, res) {
//   res.send('hello world');
// });

//addtodos
app.post('/api/todos', function (req, res) {
  var todos = new Todos();
  todos.title = req.body.title
  todos.content = req.body.content
  todos.save(function (err, rows) {
    if (err) {
      return console.log(err);
    } else {
      res.json({
        data: rows
      });
    }
  });
});
//取得所有todos
app.get('/api/todos', function (req, res) {
  Todos.find({}, function (err, rows) {
    if (err) {
      res.status(500)
        .json({
          error: err.toString()
        });
    } else {
      res.json({
        data: rows
      });
    }
  });
});
// 用id取得單一todos
app.get('/api/todos/:id', function (req, res) {
  Todos.findById({
    _id: req.params.id
  }, function (error, row) {
    //錯誤處理
    if (error) {
      // internal server error
      return res.status(500).
      json({
        error: error.toString()
      });
    }
    //資料不存在
    if (!row) {
      // data not found
      return res.status(404)
        .json({
          error: 'Not found'
        });
    }
    //有data就回傳json
    res.json({
      data: row
    });
  });
});
// 更新todos內容
app.put('/api/todos/:id', function (req, res) {
  Todos.findOne({
    _id: req.params.id
  }, function (err, foundTodos) {
    console.log(foundTodos);
    foundTodos.title = req.body.title;
    foundTodos.content = req.body.content;
    foundTodos.save(function (err, savedTodos) {
      if (err) return (err);
      return res.json({
        data: savedTodos
      });
    });
  });
});
// 移除單一筆內容
app.delete('/api/todos/:id', function (req, res) {
  Todos.findOne({
    _id: req.params.id
  }, function (err, foundTodos) {
    if (err) {
      res.send(err);
    } else {
      console.log(foundTodos);
      foundTodos.remove();
    }
  });
});

//更新toggledone
app.put('/api/todos/toggledone/:id', function (req, res) {
  Todos.findOne({
    _id: req.params.id
  }, function (err, foundTodos) {
    //foundTodos.set({ done: 1});
    if (foundTodos.done == 1) {
      foundTodos.set({
        done: 0
      });
    } else {
      foundTodos.set({
        done: 1
      });
    }
    console.log(foundTodos);
    foundTodos.save(function (err, savedTodos) {
      if (err) return (err);
      return res.json({
        data: savedTodos
      });
    });
  });
});

//網頁伺服器為http://localhost:3000
app.listen(3000, function () {
  console.log('Mongodb app listening on port 3000!');
});
module.exports = app;